(require kodhy.macros)

(import
  [numpy :as np]
  [pandas :as pd]
  [sklearn.linear-model :as sklm]
  [sklearn.ensemble :as sken]
  [kodhy.util [cbind kfold-cv-pred with-1o-interacts]])

(defn enet-f [x-train y-train x-test]
  (setv m (kwc sklm.ElasticNetCV
    :l1-ratio [.01 .1 .25 .5 .75 .9 .95 .99 1]
    :cv 10))
  (.fit m x-train y-train)
  (.predict m x-test))

(defn try-predicting [x y]

  (setv x (.as-matrix x))
  (setv y (.as-matrix y))

  (setv y-pred-ols (kfold-cv-pred x y (fn [x-train y-train x-test]
    (setv m (sklm.LinearRegression))
    (.fit m x-train y-train)
    (.predict m x-test))))
  (np.random.seed 25)
  (setv y-pred-noi (kfold-cv-pred x y enet-f))
  (np.random.seed 25)
  (setv y-pred-1oi (kfold-cv-pred (with-1o-interacts x) y enet-f))
  (np.random.seed 25)
  (setv y-pred-rf (do
    (setv m (kwc sken.RandomForestRegressor
      :n-estimators 500
      :+oob-score))
    (.fit m x y)
    m.oob-prediction-))

  (pd.concat (lc [[k yp] (pairs :constant None :ols y-pred-ols :enet y-pred-noi :enet-1oi y-pred-1oi :rf y-pred-rf)]
    (cbind :I [k]
      :MAD (np.mean (np.abs (- y (if (is yp None) (np.median y) yp))))
      :RMSE (np.sqrt (np.mean (** (- y (if (is yp None) (np.mean y) yp)) 2)))
      :ad95 (kwc np.percentile :q 95 (np.abs (- y (if (is yp None) (np.median y) yp))))))))

(defn try-predicting-MAD-each [x ys]
  (apply cbind (amap
    (kwc pd.Series ($ (try-predicting x (getl ys : it)) MAD)
      :name (slice it (len "srs_")))
    ys)))
