; -*- Hy -*-

(require kodhy.macros)

(import
  random
  json
  [math [ceil]]
  [pandas :as pd]
  [scipy.stats :as scist]
  [kodhy.util [ucfirst seq rd cbind valcounts all-unique? concat dotplot rectplot]])

(setv blueish [0. (/ 153 256) (/ 153 256)])
              ;[(/ 9 32) 0. (/ 9 16)])
(setv reddish [1. (/ 33 256) (/ 33 256)])
              ;[(/ 9 16) 0. 0.])

(defmacro cache [expr &optional [bypass 'None]]
  `(cached ~expr ~bypass (os.path.join (os.path.expanduser "~")
     ".daylight" "py-cache" "Kodi-projects" "pelt")))

(with [[o (open "../data.json")]]
  (setv data (json.load o)))

(setv demog (kwc pd.DataFrame :columns (first (get data "demog")) (slice (get data "demog") 1)))
(for [k ["parent1.ed" "parent2.ed"]]
  (setv (getl demog : k) (kwc pd.Categorical :+ordered (getl demog : k)
    ["Eighth grade or less" "Some high school" "High school graduate or GED" "Some college or post-high school" "College graduate" "Advanced graduate or professional degree"])))

(setv inst ((type (str "T") (,) {})))
(for [[k v] (.items (get data "instruments"))]
  (setattr inst k (kwc pd.DataFrame :columns (first v) (slice v 1))))

(setv famhist (get data "famhist"))
(setv dx (get data "dx"))

(setv study->subjects (get data "study_subjects"))
(setv subject->study (dict (concat
  (lc [[study subjects] (.items study->subjects)]
    (amap (, it study) subjects)))))
(setv subjects (.keys subject->study))

(setv treatment-groups (get data "treatment_groups"))

(setv s-amal (+
  (filt (= (get treatment-groups it) "CvilgT")
    (get study->subjects "Cvil"))
  (get study->subjects "Sp08")
  (get study->subjects "Sp10")))
; These subjects are missing on all posttreatment measures.
(for [s (qw 56 71 82 AG22 AH13 BN15 CC06 CG23 IW02 JH18 MD10 TO08)]
  (.remove s-amal s))
; This subject has a very large number of missing values on the pretreatment side.
(.remove s-amal "73")

(setv s-kop (list (-
  (set (get study->subjects "KoP"))
  ; Exclude subjects that we don't know the treatment conditions of.
  ; (In fact, I think they weren't put through the whole procedure.)
  (set (qw K13 K43 K44 K48)))))

(defn inst-tp [timepoint instrument cols]
  (when (string? timepoint)
    (setv timepoint {"rest" timepoint}))
  (setv d (getl (getattr inst (.replace instrument "-" "_")) :
    (+ (qw ID Timepoint) (if (string? cols) [cols] cols))))
  (setv d (.sort-index (pd.concat
    (rmap [[study x] (.groupby d (.map ($ d ID) subject->study))]
      (setv tp (.get timepoint study (get timepoint "rest")))
      (setv x (ss x (if (in tp ["Pre" "Post"])
        ; Collapse multiple pre- and post-treatment measures into
        ; means.
        (.startswith (. $Timepoint str) tp)
        (= $Timepoint tp))))
      (setv x (kwc .drop x :axis 1 "Timepoint"))
      (.mean (.groupby x "ID"))))))
  (when (= (len d.columns) 1)
    (setv d (geti d : 0)))
  (if (instance? pd.DataFrame d)
    (kwc .rename :columns (λ (+ instrument "_" it)) d)
    d))

(import pred [pred [*]])
(setv pred.treatment-groups treatment-groups)
;(import [manuscript [*]])
